﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 경계선 추적
    /// </summary>
    public static class BoundaryTracing
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성하기 - Create(pixelData)

        /// <summary>
        /// 생성하기
        /// </summary>
        /// <param name="pixelData">픽셀 데이터</param>
        /// <returns>좌표 리스트의 리스트</returns>
        public static List<List<Point>> Create(PixelData pixelData)
        {
            Size bitmapDataSize = pixelData.BitmapDataSize;

            HashSet<Point> pointHashSet = new HashSet<Point>();

            List<Point> pointList = null;

            List<List<Point>> pointListList = new List<List<Point>>();

            bool inside = false;

            int width = bitmapDataSize.Width;

            Tuple<Func<Point, Point>, int>[] neighborhoodTupleArray = new Tuple<Func<Point, Point>, int>[]
            {
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X - 1, point.Y    ), 7),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X - 1, point.Y - 1), 7),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X    , point.Y - 1), 1),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X + 1, point.Y - 1), 1),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X + 1, point.Y    ), 3),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X + 1, point.Y + 1), 3),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X    , point.Y + 1), 5),
                new Tuple<Func<Point, Point>, int>(point => new Point(point.X - 1, point.Y + 1), 5)
            };

            for(int y = 0; y < bitmapDataSize.Height; ++y)
            {
                for(int x = 0; x < bitmapDataSize.Width; ++x)
                {
                    Point point = new Point(x, y);

                    if(pointHashSet.Contains(point) && !inside)
                    {
                        inside = true;

                        continue;
                    }

                    bool isTransparent = pixelData.IsTransparentColor(point);

                    if(!isTransparent && inside)
                    {
                        continue;
                    }

                    if(isTransparent && inside)
                    {
                        inside = false;

                        continue;
                    }

                    if(!isTransparent && !inside)
                    {
                        pointListList.Add(pointList = new List<Point>());

                        pointHashSet.Add(point); pointList.Add(point);

                        int neighborCount = 1;

                        Point startPoint = point;

                        int counter1 = 0;
                        int counter2 = 0;

                        while(true)
                        {
                            Point checkPoint = neighborhoodTupleArray[neighborCount - 1].Item1(point);

                            int newNeighborCount = neighborhoodTupleArray[neighborCount - 1].Item2;

                            if(!pixelData.IsTransparentColor(checkPoint))
                            {
                                if(checkPoint == startPoint)
                                {
                                    counter1++;

                                    if(newNeighborCount == 1 || counter1 >= 3)
                                    {
                                        inside = true;

                                        break;
                                    }
                                }

                                neighborCount = newNeighborCount;

                                point = checkPoint;

                                counter2 = 0;

                                pointHashSet.Add(point);

                                pointList.Add(point);
                            }
                            else
                            {
                                neighborCount = 1 + (neighborCount % 8);

                                if(counter2 > 8)
                                {
                                    counter2 = 0;

                                    pointList = null;

                                    break;
                                }
                                else
                                {
                                    counter2++;
                                }
                            }
                        }
                    }
                }
            }

            return pointListList;
        }

        #endregion
        #region 좌표 리스트 구하기 - GetPointList(pointListList)

        /// <summary>
        /// 좌표 리스트 구하기
        /// </summary>
        /// <param name="pointListList">좌표 리스트 리스트</param>
        /// <returns>좌표 리스트</returns>
        public static List<Point> GetPointList(List<List<Point>> pointListList)
        {
            pointListList.Sort((x, y) => x.Count.CompareTo(y.Count));

            return pointListList.Last();
        }

        #endregion
    }
}