﻿using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 픽셀 데이터 2
    /// </summary>
    public class PixelData2 : PixelData
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 비트맵
        /// </summary>
        private readonly Bitmap bitmap;

        /// <summary>
        /// 비트맵 데이터
        /// </summary>
        private readonly BitmapData bitmapData;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PixelData2(bitmap, bitmapData)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="bitmap">비트맵</param>
        /// <param name="bitmapData">비트맵 데이터</param>
        public PixelData2(Bitmap bitmap, BitmapData bitmapData) : base(bitmapData)
        {
            this.bitmap = bitmap;

            this.bitmapData = bitmapData;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성하기 - Create(bitmap)

        /// <summary>
        /// 생성하기
        /// </summary>
        /// <param name="bitmap">비트맵</param>
        /// <returns>픽셀 데이터 2</returns>
        public static new PixelData2 Create(Bitmap bitmap)
        {
            BitmapData bitmapData = GetBitmapData(bitmap, false);

            return new PixelData2(bitmap, bitmapData);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 저장하기 - Save(targetPath)

        /// <summary>
        /// 저장하기
        /// </summary>
        /// <param name="targetPath">타겟 경로</param>
        public void Save(string targetPath)
        {
            Marshal.Copy(pixelArray, 0, this.bitmapData.Scan0, pixelCount);

            this.bitmap.UnlockBits(this.bitmapData);

            this.bitmap.Save(targetPath);
        }

        #endregion
        #region 비트맵 구하기 - GetBitmap()

        /// <summary>
        /// 비트맵 구하기
        /// </summary>
        /// <returns>비트맵</returns>
        public Bitmap GetBitmap()
        {
            Marshal.Copy(pixelArray, 0, this.bitmapData.Scan0, pixelCount);

            this.bitmap.UnlockBits(this.bitmapData);

            return this.bitmap;            
        }

        #endregion
        #region 색상 설정하기 - SetColor(point, argb)

        /// <summary>
        /// 색상 설정하기
        /// </summary>
        /// <param name="point">좌표</param>
        /// <param name="argb">ARGB 값</param>
        public void SetColor(Point point, int argb)
        {
            base[point] = argb;
        }

        #endregion
    }
}